#
# IBM Confidential
# OCO Source Materials
# Rational ClearQuest
# (C) Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

use strict;

use File::Copy;
use HTML::Entities;

sub Usage {
	print <<EOF;

Please see the IBM Rational ClearQuest Information Center for usage details of ClearQuest EmailRelay application.

EOF
	exit 1;
}

# See if they requested usage info...
if (($#ARGV < 1) || ($ARGV[0] =~ /^-[h\?]/i)) {
	Usage();
}

my $Q = "";
my $gIsWindows = 0;
my $ext = ".sh";
my $gCaptureExeOutputData = "";
my $gEmailRelayWASProfileName = "";
my $gClearQuestHomeDir = "";
my $gWASAppSrvBinDir = "";
my @gExistingProfileNames;
my $gWASaccount = "";


##############################################################################
# Main entery to cqemailrelay.pl script
if (($ARGV[0] =~ /^-emailrelayapp/i) || ($ARGV[2] =~ /^-emailrelayapp/i)) {
	# This is where we execute cqemailrelay.pl (to configure web.xml)
	if (($#ARGV < 3)) {
		Usage();
	}
	
	my $relaycfgdir = "";
	my $emailrelayapp = "";
	my $haveUser = 0;
	my $havePassword = 0;
	
	my $index = 0;
	foreach my $argnum (0 .. $#ARGV) {
		if ($ARGV[$index] =~ /^-emailrelayapp/i) {
			$emailrelayapp = $ARGV[$index + 1];
		}
		if ($ARGV[$index] =~ /^-relaycfgdir/i) {
			$relaycfgdir = $ARGV[$index + 1];
		}
		if ($ARGV[$index] =~ /^-user/i) {
			$gWASaccount .= " -user ".$ARGV[$index + 1]." ";
			$haveUser = 1;
		}
		if ($ARGV[$index] =~ /^-password/i) {
			$gWASaccount .= " -password ".$ARGV[$index + 1]." ";
			$havePassword = 1;
		}
		$index++;
	}
	
	if (($emailrelayapp eq "") || ($relaycfgdir eq "")) {
		Usage();
	}
	
	if ((($haveUser == 0) && ($havePassword == 1)) || (($haveUser == 1) && ($havePassword == 0))) {
		Usage();
	}
	
	$emailrelayapp = lc($emailrelayapp);
	
	if (($emailrelayapp =~ /start/i) || ($emailrelayapp =~ /stop/i)) {	
		my $relayCfgFile = getEmailRelayCfgFile($relaycfgdir);
		if (-f $relayCfgFile) {
			updateWebXmlFile($relayCfgFile, $emailrelayapp);
		}
		else {
			die "\nThe ClearQuest EmailRelay configuration file does not exist: ".$relayCfgFile."\n";
		}
	}
	else {
		die "\n'-emailrelayapp' must be 'start' or 'stop'.\n";
	}
}
else {
	# This is where we execute cqemailrelay.jar (to configure cqemailrelay_cfg.xml)
	my @para;
	my $index = 0;
	
	foreach my $argnum (0 .. $#ARGV) {
		$para[$index] = "\"$ARGV[$index]\"";
		$index++;
	}
	
	my $classpath;
	my $java;
	
	if ($ENV{'OS'} =~ /Windows/i) {
		$classpath = './cqemailrelay.jar;';
		$java = $ENV{'JRE_HOME'}.'\bin\java.exe';
	}
	else {
		$classpath = './cqemailrelay.jar:';
		$java = $ENV{'CQ_HOME'}.'/../common/java/jre/bin/java';
	}
	
	my $cmd = "\"$java\" -version";
	my $cmdout = `$cmd 2>&1`;
	
	# Check return code
	if ($? != 0) {
		die "\nCannot find executable Java\nPlease check if ClearQuest is installed correctly.\n";
	}
	
	# Verify java version
	my $version6 = 0;
	if ($cmdout =~ /^java version "(\d+)\.(\d+)\./) {
		if ($1 > 1) {
			$version6 = 1;
		}
		elsif (($1 == 1) && ($2 >= 6)) {
			$version6 = 1;
		}
	}
	if ($version6 == 0) {
		die "\nThis application requires Java version 1.6 or greater.\n";
	}
	
	my $cmd = "\"$java\" -classpath $classpath com.ibm.rational.clearquest.emailrelay.common.ConfigGenerator @para";
	
	system($cmd);
}

exit 0;


##############################################################################
##############################################################################


##############################################################################
# Update CQ's EmailRelay application, set it to 'start' or 'stop' and point it
# to the EmailRelay configuration file
sub updateWebXmlFile
{
	my ($relayCfgFile, $emailrelayapp) = @_;
	
	my $emailRelayAppPath = "";
	my $webXMLFile = "";
	my $siteName = "";
	
	setGlobalVars();	# this is to set the above 3 variables
	
	($webXMLFile, $emailRelayAppPath) = findEmailRelayWebXmlFile();
	if ($webXMLFile eq "") {
		die "\nERROR: Cannot find ClearQuest EmailRelay web.xml file.  Make sure the WebSphere profile in which you have deployed ClearQuest Web is started.\n";
	}
	
	$siteName = findReplicaName($relayCfgFile);
	if ($siteName eq "") {
		die "\nERROR: Cannot determine site-name from the file: ".$relayCfgFile."\n";
	}
	
	print "\n\nAttempting to update ClearQuest EmailRelay application file: ".$webXMLFile."\n";
	
	my $line;
	my $data;
	if (!open(FH, $webXMLFile)) {
		die "\nERROR: Cannot open file: ".$webXMLFile."\n";
	}
	while ($line = <FH>) {
		$data .= $line;
	}
	close FH;
	
	my $newValue;
	
	# First point to where ever is the CQ EmailRelay's cqemailrelay_cfg.xml file is
	$newValue = HTML::Entities::encode($relayCfgFile);
	$data =~ s/<param-value>\S*\s*\D*\d*CQ-\S*\s*\D*\d*.xml<\/param-value>/<param-value>$newValue<\/param-value>/s;
	$data =~ s{
		(<param-name>config</param-name>)	# $1 = "sitename" element
		(.*?)								# $2 = stuff between elements
		(<param-value>)						# $3 = param-value start tag
		.*?									# discard param-value content
		(</param-value>)					# $4 = param-value end tag
	}{$1$2$3$newValue$4}sgx;
	
	# Next set the site-name
	$newValue = HTML::Entities::encode($siteName);
	$data =~ s{
		(<param-name>sitename</param-name>) # $1 = "sitename" element
		(.*?)								# $2 = stuff between elements
		(<param-value>)						# $3 = param-value start tag
		.*?									# discard param-value content
		(</param-value>)					# $4 = param-value end tag
	}{$1$2$3$newValue$4}sgx;
	
	# Finaly, enable or disable CQ EmailRelay
	if ($emailrelayapp eq "start") {
		$newValue = "<load-on-startup>1</load-on-startup>";
		$data =~ s/<!--\s*<load-on-startup>1<\/load-on-startup>\s*-->/$newValue/sm;
	}
	elsif ($emailrelayapp eq "stop") {
		# if it's already stopped, do nothing
		if ($data =~ /<!--\s*<load-on-startup>1<\/load-on-startup>\s*-->/) {
			print "\nClearQuest EmailRelay is already stopped.\n";
		}
		else {
			$newValue = "\n    <!--\n    <load-on-startup>1</load-on-startup>\n    -->"."\n";
			$data =~ s/\s*<load-on-startup>1<\/load-on-startup>\s*/$newValue/sm;
		}
	}
	
	my $status = safeFileUpdate($webXMLFile, $data);	# now update web.xml file
	
	# If we are starting EmailRelay, we also want to set PARENT_LAST as our classloader
	if (($status eq 1) and ($emailrelayapp eq "start")) {
		print "\n\nSetting PARENT_LAST on ClearQuest EmailRelay application.\n";
		$status = setWASParentLast($emailRelayAppPath);
	}
	
	if ($status eq 1) {
		
		# $status = restartWASApp($gEmailRelayWASProfileName, "cqemailrelay", $emailrelayapp);
		
		print "\n\n---------------------------------------------------------\n";
		
		if ($status ne 1) {
			print "\nThere was an error while attempting to restart the WebSphere application 'cqemailrealy' on the WAS profile '".$gEmailRelayWASProfileName."'.\n";
			print "\nYou many need to intervene and manually restart the WAS profile '".$gEmailRelayWASProfileName."'.\n";
		}
		else {
			print "\nYour ClearQuest EmailRelay application has been updated successfully.\n";
			
			# This next line would no longer be needed once 'cqemailrelay' application can be successfully be restarted via restartWASApp()
			print "\nAttention: You must restart the WebSphere profile '".$gEmailRelayWASProfileName."' for the changes to take effect.\n";
		}
	}
	
	print "\n";
}

##############################################################################
# Find and update CQ EmailRelay's web.xml file
sub findEmailRelayWebXmlFile
{
	my $emailRelayWebXmlFile = "";
	
	my $emailRelayWASAppSrvBinDir = "";
	
	# Get a list of WAS profiles on this machine
	getListOfProfilesName();
	foreach my $line ( @gExistingProfileNames ) {
		$line = trim($line);
		
		my $emailRelayAppPath = "";
		my $path = getProfilePath($line);
		my $emailRelayApp = findEmailRelayWASApp($path."bin/");
		
		if ($emailRelayApp ne "") {
			# Working on: WebSphere:name=cqemailrelay,process=server1,platform=dynamicproxy,node=xxxxNode01,J2EEName=cqemailrelay,Server=server1,version=8.0.0.3,type=Application,mbeanIdentifier=cells/xxxxNode01Cell/applications/cqemailrelay.ear/deployments/cqemailrelay/deployment.xml#ApplicationDeployment_1337883272440,cell=xxxxNode01Cell,spec=1.0
			my @data = split(/,/, $emailRelayApp);
			foreach my $item ( @data ) {
				if ($item =~ m/mbeanIdentifier=/) {
					$emailRelayAppPath = $item;
					last;
				}
			}
		}
		
		if ($emailRelayAppPath ne "") {
			# Working on: mbeanIdentifier=cells/xxxxNode01Cell/applications/cqemailrelay.ear/deployments/cqemailrelay/deployment.xml#ApplicationDeployment_1337883272440
			my @data = split(/=/, $emailRelayAppPath);
			if (scalar (@data) ne 2) {
				die "\nERROR: Unexepcted data while looking for CQ EmailRelay application: ".$emailRelayAppPath."\n";
			}
			
			$emailRelayAppPath = @data[1];
			
			# Working on: cells/xxxxNode01Cell/applications/cqemailrelay.ear/deployments/cqemailrelay/deployment.xml#ApplicationDeployment_1337883272440
			my $foundAt = rindex($emailRelayAppPath, "/");	# remove: deployment.xml#ApplicationDeployment_1337883272440
			if ($foundAt ne -1) {
				# we should have: cells/xxxxNode01Cell/applications/cqemailrelay.ear/deployments/cqemailrelay
				$emailRelayAppPath = trim(substr($emailRelayAppPath, 0, $foundAt));
				$emailRelayWebXmlFile = $emailRelayAppPath."/cqemailrelay.war/WEB-INF/web.xml";
			}
			
			$emailRelayWASAppSrvBinDir = $path."bin/";	# Assume this is out EmailRelay's BIN path
			
			# See that we have our web.xml file where it's suppose to be
			$path = $path."config/".$emailRelayWebXmlFile;
			if (-f $path) {
				$emailRelayWebXmlFile = $path;
				$gEmailRelayWASProfileName = $line;
				last;
			}
			else {
				$emailRelayWASAppSrvBinDir = "";	# Our assumption was wrong; reset it
			}
		}
	}
	
	return ($emailRelayWebXmlFile, $emailRelayWASAppSrvBinDir);
}

##############################################################################
# Look for CQ EmailRelay application location
sub findEmailRelayWASApp
{
	my ($wasBinDir) = @_;
	
	createWASAppFinderScript();
	
	my $cmd = $Q.$wasBinDir."wsadmin".$ext.$Q.$gWASaccount." -lang jython -f ".$Q.$gClearQuestHomeDir."wasAppFinder.py".$Q;
	
	executeExternalCommand($cmd, 1);
	
	my $emailRelayApp = "";
	my @data = split(/\n/, $gCaptureExeOutputData);
	foreach my $line ( @data ) {
		if ($line =~ m/\[WSAppInfo\]/) {
			if ($line =~ m/name=cqemailrelay/) {
				$emailRelayApp = $line;
				last;
			}
		}
	}
	
	deleteWASAppFinderScript();
	
	return $emailRelayApp;
}

##############################################################################
# For cqemailrelay application, set PARENT_LAST for classloader
sub setWASParentLast
{
	my ($wasBinDir) = @_;
	
	createWASParentLastSetterScript();
	
	my $cmd = $Q.$wasBinDir."wsadmin".$ext.$Q.$gWASaccount." -lang jython -f ".$Q.$gClearQuestHomeDir."wasParentLastSetter.py".$Q;
	
	executeExternalCommand($cmd, 1);
	
	my $status = 0;
	my @data = split(/\n/, $gCaptureExeOutputData);
	foreach my $line ( @data ) {
		if ($line =~ m/SUCCESS: PARENT_LAST was set for application: cqemailrelay/) {
			$status = 1;
			last;
		}
	}
	
	deleteWASParentLastSetterScript();
	
	return $status;
}

##############################################################################
# Restarts the cqemailrelay application
sub restartWASApp
{
	my ($profileName, $appName, $startOrStop) = @_;
	
	my $doStop = 0;
	my $doStart = 0;
	
	if ($startOrStop eq "start") {
		# Since web.xml was changed, for the 'start' to take effect, we need to stop (in case it's already started) and then start cqemailrelay application
		$doStop = 1;
		$doStart = 1;
	}
	
	if ($startOrStop eq "stop") {
		# Stop cqemailrelay application
		$doStop = 1;
		$doStart = 0;
	}
	
	createWASAppStartStopScript();
	
	my $status = 0;
	
	# Get a list of WAS profiles on this machine
	getListOfProfilesName();
	foreach my $line ( @gExistingProfileNames ) {
		$line = trim($line);
		
		my $emailRelayAppPath = "";
		my $path = getProfilePath($line);
		my $emailRelayApp = findEmailRelayWASApp($path."bin/");
		
		if ($emailRelayApp ne "") {
			my $wasBinDir = $path."bin/";
			my $cmd = "";
			
			if ($doStop == 1) {
				# Stop the application
				$cmd = $Q.$wasBinDir."wsadmin".$ext.$Q.$gWASaccount." -profileName ".$profileName." -lang jython -f ".$Q.$gClearQuestHomeDir."stopStartWASApp.py".$Q." -action:stop ".$appName;
				executeExternalCommand($cmd, 1);
			
				my @data = split(/\n/, $gCaptureExeOutputData);
				foreach my $line ( @data ) {
					if ($line =~ m/Stopped SUCCESSFULLY/) {
						$status = 1;
						last;
					}
				}
			}
			
			if ($doStart == 1) {
				# start the application
				$cmd = $Q.$wasBinDir."wsadmin".$ext.$Q.$gWASaccount." -profileName ".$profileName." -lang jython -f ".$Q.$gClearQuestHomeDir."stopStartWASApp.py".$Q." -action:start ".$appName;
				executeExternalCommand($cmd, 1);
			
				my @data = split(/\n/, $gCaptureExeOutputData);
				foreach my $line ( @data ) {
					if ($line =~ m/Started SUCCESSFULLY/) {
						$status = 1;
						last;
					}
				}
			}
						
			last;
		}
	}
	
	deleteWASAppStartStopScript();
	
	return $status;
}

##############################################################################
# Returns the list of profiles name
sub getListOfProfilesName
{
	my $status = 1;
	
	$gCaptureExeOutputData = "";		# remove any old output data
	my $cmd = $Q.$gWASAppSrvBinDir."manageprofiles".$ext.$Q." -listProfiles";
	
	executeExternalCommand($cmd, 1);
	
	# validate the output of executing the above command
	# expected data: [AppSrv01, cqweb, cqsearchprofile]
	
	# Remove the leading "[" and trainling "]"
	$gCaptureExeOutputData = trim($gCaptureExeOutputData);
	my $foundAt;
	$foundAt = index($gCaptureExeOutputData, "[");
	if ($foundAt ne 0) {
		# We have more than one line; something went wrong
		die "\nERROR: Unexepcted data while looking for list of WAS profile names: ".$gCaptureExeOutputData."\n";
	}
	else {
		$gCaptureExeOutputData = substr($gCaptureExeOutputData, 1);
	}
	$foundAt = rindex($gCaptureExeOutputData, "]");
	if ($foundAt ne length($gCaptureExeOutputData) - 1) {
		# We have more than one line; something went wrong
		die "\nERROR: Unexepcted data while looking for list of WAS profile names: ".$gCaptureExeOutputData."\n";
	}
	else {
		$gCaptureExeOutputData = substr($gCaptureExeOutputData, 0, length($gCaptureExeOutputData) - 1);
	}
	
	# iterate of the remaining data which is now comma delimited
	my @data = split(/,/, $gCaptureExeOutputData);
	foreach my $item ( @data ) {
		push(@gExistingProfileNames, trim($item));
	}
}

##############################################################################
# Get a profile's path
sub getProfilePath
{
	my ($profileName) = @_;
	
	my $profilePath = "";
	my $cmd = $Q.$gWASAppSrvBinDir."manageprofiles".$ext.$Q." -getPath -profileName ".$profileName;
	
	executeExternalCommand($cmd, 1);
	
	my $numOfLines = 0;
	my @data = split(/\n/, $gCaptureExeOutputData);
	foreach my $line ( @data ) {
		if ($numOfLines ne 0) {
			die "\nERROR: Unexepcted data while looking for list of WAS profile path: ".$gCaptureExeOutputData."\n";
		}
		$profilePath = $line."/";
		$profilePath =~ s/\\/\//sg;
		$numOfLines++;
	}
	
	return $profilePath;
}

##############################################################################
# Executes an external program / command; checks for it's success / completion, 
# and terminates if it fails (if instructed to)
sub executeExternalCommand
{
	my ($cmd, $captureOutput) = @_;
	
	if ($captureOutput == 1) {
		$gCaptureExeOutputData = "";
	}
	
	print "Executing: system -> [".$cmd."]"."\n";
	
	# execute the command
	if (open(FH, "$cmd 2>&1 |")) {
		my $line;
		
		while ($line = <FH>) {
			print $line;
			if ($captureOutput == 1) {
				$gCaptureExeOutputData .= $line;
			}
		}
		close FH;
	}
	else {
		die "\nERROR: Cannot execute: ".$cmd."\n\n";
	}
}

##############################################################################
# Find where WebSphere is installed
sub findWASAppServerDir
{
	# IM's WAS AppServer file is in: %RATIONAL_HOME%\was_info.txt"
	my $appSrvFile = $gClearQuestHomeDir."was_info.txt";
	my $foundAppSrvPath = "";
	
	if (-f $appSrvFile) {
		my @data;
		if (open(FH, $appSrvFile)) {
			@data = <FH>;
			close(FH);
		}
		
		# we are looking for one of those two lines in the file 'was_info.txt':
		#   WAS_HOME=C:\Program Files\IBM\WebSphere\AppServer
		#   WAS_PROFILE=C:\Program Files\IBM\WebSphere\AppServer\profiles\AppSrv01
		
		my $line;
		foreach $line (@data) {
			$line = trim($line);
			
			if ($line =~ m/WAS_HOME=/) {
				# we found: C:\Program Files\IBM\WebSphere\AppServer
				$line =~ s/WAS_HOME=//;
				$foundAppSrvPath = trim($line);
			}
			elsif ($line =~ /WAS_PROFILE=/) {
				# we found: WAS_PROFILE=C:\Program Files\IBM\WebSphere\AppServer\profiles\AppSrv01
				$line =~ s/WAS_PROFILE=//;
				$foundAppSrvPath = trim($line);
			}
			
			# do we have our path, and is it the correct one?
			if (($foundAppSrvPath ne "") and (-f $foundAppSrvPath."/bin/manageprofiles".$ext)) {
				last;
			}
			else {
				$foundAppSrvPath = "";	# keep looking or we haven't found it yet
			}
		}
	}
	
	if ($foundAppSrvPath eq "") {
		die "\nERROR: Cannot determine WebSphere's AppServer home. Please see the IBM Rational ClearQuest Information Center on how to set it via 'CQER_AppServer_HOME' environment variable.\n";
	}
	
	return $foundAppSrvPath;
}

##############################################################################
# Figure out where CQ is installed
sub getClearQuestHomeDir
{
	my $cqHomeDir = "";
	
	$cqHomeDir = $ENV{'CLEARQUEST_HOME'};	# on Windows
	if (!defined($cqHomeDir) or $cqHomeDir eq "") {
		# don't give up just yet
		$cqHomeDir = $ENV{'CQ_HOME'};	# on Unix
		if (!defined($cqHomeDir) or $cqHomeDir eq "") {
			# keep trying, one last time ...
			$cqHomeDir = $ENV{'RATIONAL_HOME'};
			if (defined($cqHomeDir) and $cqHomeDir ne "") {
				# 'RATIONAL_HOME' doesn't have 'ClearQuest"
				$cqHomeDir .= "/ClearQuest";
			}
		}
	}
	if ((substr($cqHomeDir, -1) ne "/") or (substr($cqHomeDir, -1) ne "\\")) {
		$cqHomeDir .= "/";
	}
	
	$cqHomeDir =~ s/\\/\//sg;
	
	return $cqHomeDir;
}

##############################################################################
# Figure out where WAS is installed
sub getWASAppSrvBinDir
{
	my $wasHomeDir = "";
	
	$wasHomeDir = $ENV{'CQER_AppServer_HOME'}; ### ###
	if (!defined($wasHomeDir) or $wasHomeDir eq "") {
		# we need to get the AppServer's path from IM's file
		$wasHomeDir = findWASAppServerDir();
	}
	if (!((substr($wasHomeDir, -1) eq "/") or (substr($wasHomeDir, -1) eq "\\"))) {
		$wasHomeDir .= "/";
	}
	
	$wasHomeDir =~ s/\\/\//sg;
	
	if (!((substr($wasHomeDir, -1) eq "/") or (substr($wasHomeDir, -1) eq "\\"))) {
		$wasHomeDir .= "/";
	}
	
	$wasHomeDir = $wasHomeDir."bin/";
	
	if (!(-d $wasHomeDir)) {
		die "\nERROR: Cannot find WebSphere application at: ".$wasHomeDir."\n";
	}
	
	return $wasHomeDir;
}

##############################################################################
# cleans up and normalizes the WAS App dir
sub getEmailRelayCfgFile
{
	my ($file) = @_;
	
	$file =~ s/\\/\//sg;
	if (!((substr($file, -1) eq "/") or (substr($file, -1) eq "\\"))) {
		$file .= "/";
	}
	
	return $file."cqemailrelay_cfg.xml";
}

##############################################################################
# Updates a file in a safe way; it will make a backup of the current file, 
# write to it, and delete the backup file.
sub safeFileUpdate
{
	my ($fileToUpdate, $data) = @_;
	
	my $backupFile = $fileToUpdate."backup";
	my $status = 0;		# assume we will fail
	
	# first make sure the file exist
	if (!(-f $fileToUpdate)) {
		die "\nERROR: Cannot find file to update: ".$fileToUpdate."\n";
	}
	else {
		# make a backup copy of the current file (the one we will be modifying)
		if (!copy($fileToUpdate, $backupFile)) {
			die "\nERROR: Cannot create a backup of: ".$fileToUpdate." as: ".$backupFile."\n";
		}
		else {
			# now open the existing file and write to it the new data
			if (open(FH, ">$fileToUpdate")) {
				print FH $data;		# write to it and close it
				close FH;
				$status = 1;	# all is well
			}
			else {
				die "\nERROR: Cannot create write to file: ".$fileToUpdate."\n";
				# restore from the backup file
				if (!copy($backupFile, $fileToUpdate)) {
					die "\nERROR: Cannot restore original file from: ".$backupFile." from: ".$fileToUpdate."\n"."\nAttention: You need to restore the backup file: ".$$backupFile." manualy\n";
				}
			}
			
			# delete the backup file
			if (unlink($backupFile) ne 1) { # returns the number of files deleted (1 in our case)
				warn "\nCannot delete temporary backup file: ".$backupFile."\n";
			}
		}
	}
	
	return $status;
}

##############################################################################
# Find out what is the replica name
sub findReplicaName
{
	my ($relayCfgFile) = @_;
	
	my $replicaName = "";
	
	my @data;
	if (open(FH, $relayCfgFile)) {
		@data = <FH>;
		close(FH);
		
		my $line;
		foreach $line (@data) {
			$line = trim($line);
			# try to match: <CQReplica sitename="&lt;local&gt;">
			if ($line =~ /(<)(.*?)(CQReplica)(.*?)(sitename)(.*?)(=)(.*?)(")(.+)(")(.*?)(>)+/) {
				$replicaName = trim(HTML::Entities::decode($10));
				last;
			}
		}
	}
	
	return $replicaName;
}

##############################################################################
# Removes white characters from start and end of a string
sub trim
{
	my ($string) = @_;
	
	if (defined($string)) {
		$string =~ s/^\s+//;
		$string =~ s/\s+$//;
	}
	
	return $string;
}

##############################################################################
# Creates the wasAppFinder.py WebSphere script
sub createWASAppFinderScript
{
	my $scriptData = <<EOF;
#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2012.  All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#

#----------------------------------------------------------------
# get line separator 
import  java.lang.System  as  sys

lineSeparator = sys.getProperty('line.separator')
cells = AdminConfig.list('Cell').split(lineSeparator)

def wasAdminToList(inStr):
	inStr = inStr.rstrip()
	outList = []
	if (len(inStr) > 0 and inStr[0] == '[' and inStr[-1] == ']'):
		tmpList = inStr[1:-1].split(" ")
	else:
		tmpList = inStr.split("\\n")	#splits for Windows or Linux
	for item in tmpList:
		item = item.rstrip()			#removes any Windows "\\r"
		if (len(item) > 0):
			outList.append(item)
	return outList
#endDef

for cell in cells:
	print "\\n\\ncell: " + cell + "\\n"
	#-----------------------------------------------------------------
	nodes = AdminConfig.list('Node', cell).split(lineSeparator)
	for node in nodes:
		print "node: " + node + "\\n"
		#--------------------------------------------------------------
		cname = AdminConfig.showAttribute(cell, 'name')
		nname = AdminConfig.showAttribute(node, 'name')
		servs = AdminControl.queryNames('type=Server,cell=' + cname + ',node=' + nname + ',*')
		servs = wasAdminToList(servs)
		print "Number of running servers on node " + nname + ": %s \\n" % (len(servs))
		for server in servs:
			#---------------------------------------------------------
			print "server: " + server + "\\n"
			sname = AdminControl.getAttribute(server, 'name')
			ptype = AdminControl.getAttribute(server, 'processType')
			pid   = AdminControl.getAttribute(server, 'pid')
			state = AdminControl.getAttribute(server, 'state')
			jvm = AdminControl.queryNames('type=JVM,cell=' + cname + ',node=' + nname + ',process=' + sname + ',*')
			osname = AdminControl.invoke(jvm, 'getProperty', 'os.name')
			print " " + sname + " " +  ptype + " has pid " + pid + "; state: " + state + "; on " + osname + "\\n"
			#---------------------------------------------------------
			apps = AdminControl.queryNames('type=Application,cell=' + cname + ',node=' + nname + ',process=' + sname + ',*').split(lineSeparator)
			print "Number of applications running on " + sname + ": %s \\n" % (len(apps))
			for app in apps:
				print "[WSAppInfo] " + app + "\\n"
			print "\\n"
EOF
	
	eval {
		my $theFile = $gClearQuestHomeDir."wasAppFinder.py";
		
		if (!open(FH, ">$theFile")) {
			die "\nERROR: Cannot create file: ".$theFile."\n";
		}
		print FH $scriptData;
		close FH;
	};
}

##############################################################################
# Deletes the wasAppFinder.py WebSphere script
sub deleteWASAppFinderScript
{
	my $theFile = $gClearQuestHomeDir."wasAppFinder.py";
	
	if (-e $theFile) {
		if (unlink($theFile) ne 1) {
			warn "Cannot delete no longer needed file: ".$theFile."\n";
		}
	}
}
##############################################################################
# Creates the wasParentLastSetter.py WebSphere script
sub createWASParentLastSetterScript
{
	my $scriptData = <<EOF;
#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2012.  All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#

import  java.lang.System  as  sys

def wasAdminToList(inStr):
	inStr = inStr.rstrip()
	outList = []
	if (len(inStr) > 0 and inStr[0] == '[' and inStr[-1] == ']'):
		tmpList = inStr[1:-1].split(" ")
	else:
		tmpList = inStr.split("\\n")	#splits for Windows or Linux
	for item in tmpList:
		item = item.rstrip()		#removes any Windows "\\r"
		if (len(item) > 0):
			outList.append(item)
	return outList
#endDef

def getWebModule(config, applicationName):
	webModules = config.list('WebModuleDeployment')
	webModules = wasAdminToList(webModules)
	for webModule in webModules:
		if (webModule.find(applicationName) != -1):
			return webModule
	return None
#endDef

applicationName = "cqemailrelay"
webModule = getWebModule(AdminConfig, applicationName)
if (webModule != None):
	AdminConfig.modify(webModule, "[[classloaderMode PARENT_LAST]]")
	AdminConfig.save()
	print "SUCCESS: PARENT_LAST was set for application: " + applicationName
else:
	print "ERROR: Cannot find web module for application: " + applicationName

EOF
	
	eval {
		my $theFile = $gClearQuestHomeDir."wasParentLastSetter.py";
		
		if (!open(FH, ">$theFile")) {
			die "\nERROR: Cannot create file: ".$theFile."\n";
		}
		print FH $scriptData;
		close FH;
	};
}

##############################################################################
# Deletes the wasParentLastSetter.py WebSphere script
sub deleteWASParentLastSetterScript
{
	my $theFile = $gClearQuestHomeDir."wasParentLastSetter.py";
	
	if (-e $theFile) {
		if (unlink($theFile) ne 1) {
			warn "Cannot delete no longer needed file: ".$theFile."\n";
		}
	}
}

##############################################################################
# Creates the stopStartWASApp.py WebSphere script
sub createWASAppStartStopScript
{
	my $scriptData = <<EOF;
#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2012.  All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#

import os, sys, java, string

lineSeparator = java.lang.System.getProperty('line.separator')
cells = AdminConfig.list('Cell').split(lineSeparator)

#-----------------------------------------------------------------
# Start an application
def startApp(appName):
	print "Starting: " + appName + "\\n"
	
	for cell in cells:
		print "\\n\\ncell: " + cell + "\\n"
		#-----------------------------------------------------------------
		nodes = AdminConfig.list('Node', cell).split(lineSeparator)
		for node in nodes:
			print "node: " + node + "\\n"
			cname = AdminConfig.showAttribute(cell, 'name')
			nname = AdminConfig.showAttribute(node, 'name')
			
			appManager = AdminControl.queryNames('cell=' + cname + ',node=' + nname +',type=ApplicationManager,process=server1,*')
			AdminControl.invoke(appManager, 'startApplication', appName)
			
			print "Started SUCCESSFULLY\\n"
			
			break

#-----------------------------------------------------------------
# Stop an application
def stopApp(appName):
	print "Stopping: " + appName
	
	for cell in cells:
		print "\\n\\ncell: " + cell + "\\n"
		nodes = AdminConfig.list('Node', cell).split(lineSeparator)
		for node in nodes:
			cname = AdminConfig.showAttribute(cell, 'name')
			nname = AdminConfig.showAttribute(node, 'name')
			
			appManager = AdminControl.queryNames('cell=' + cname + ',node=' + nname +',type=ApplicationManager,process=server1,*')
			AdminControl.invoke(appManager, 'stopApplication', appName)
			
			print "Stopped SUCCESSFULLY\\n"
			
			break

#-----------------------------------------------------------------
# Handle request
def startStopApp(action, appName):
	if action == "-action:start":
		startApp(appName)
	elif action == "-action:stop":
		stopApp(appName)
	else:
		print "Unknown argument: " + action
		sys.exit(101)

#-----------------------------------------------------------------
# Main
if (len(sys.argv) != 2):
    print "This script will start or stop a WebSphere application"
    print ""
    print "This script requires 2 parameter: "
    print "	  1) -action:[start | stop]"
    print "	  2) appName"
    print ""
    print "e.g.:     startStopApp -action:start cqemailrelay"
else:
    startStopApp(sys.argv[0], sys.argv[1])

EOF

	eval {
		my $theFile = $gClearQuestHomeDir."stopStartWASApp.py";
		
		if (!open(FH, ">$theFile")) {
			die "\nERROR: Cannot create file: ".$theFile."\n";
		}
		print FH $scriptData;
		close FH;
	};
}

##############################################################################
# Deletes the stopStartWASApp.py WebSphere script
sub deleteWASAppStartStopScript
{
	my $theFile = $gClearQuestHomeDir."stopStartWASApp.py";
	
	if (-e $theFile) {
		if (unlink($theFile) ne 1) {
			warn "Cannot delete no longer needed file: ".$theFile."\n";
		}
	}
}

##############################################################################
# Sets any global variables that must be set before we can use the script
sub setGlobalVars
{
	$Q = "\'";	# assume Unix
	$gIsWindows = 0;
	if ($^O =~ /MSWin/) {
		$gIsWindows = 1;
		$Q = "\"";
	}
	
	$ext = ($gIsWindows eq 1) ? ".bat" : ".sh";
	
	$gClearQuestHomeDir = getClearQuestHomeDir();
	$gWASAppSrvBinDir = getWASAppSrvBinDir();
}
